@@ -5,7 +5,7 @@
  * This file is part of Libgcrypt
  *
  * Libgcrypt is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
+ * it under the terms of the GNU Lesser general Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
@@ -15,8 +15,8 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://www.gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 /* This implementation was written by Nikos Mavroyanopoulos for GNUTLS
@@ -35,7 +35,6 @@
 #include "g10lib.h"
 #include "types.h"
 #include "cipher.h"
-#include "cipher-internal.h"
 
 #define RFC2268_BLOCKSIZE 8
 
@@ -137,13 +136,6 @@ do_encrypt (void *context, unsigned char *outbuf, const unsigned char *inbuf)
   outbuf[7] = word3 >> 8;
 }
 
-static unsigned int
-encrypt_block (void *context, unsigned char *outbuf, const unsigned char *inbuf)
-{
-  do_encrypt (context, outbuf, inbuf);
-  return /*burn_stack*/ (4 * sizeof(void *) + sizeof(void *) + sizeof(u32) * 4);
-}
-
 static void
 do_decrypt (void *context, unsigned char *outbuf, const unsigned char *inbuf)
 {
@@ -196,13 +188,6 @@ do_decrypt (void *context, unsigned char *outbuf, const unsigned char *inbuf)
   outbuf[7] = word3 >> 8;
 }
 
-static unsigned int
-decrypt_block (void *context, unsigned char *outbuf, const unsigned char *inbuf)
-{
-  do_decrypt (context, outbuf, inbuf);
-  return /*burn_stack*/ (4 * sizeof(void *) + sizeof(void *) + sizeof(u32) * 4);
-}
-
 
 static gpg_err_code_t
 setkey_core (void *context, const unsigned char *key, unsigned int keylen, int with_phase2)
@@ -228,9 +213,6 @@ setkey_core (void *context, const unsigned char *key, unsigned int keylen, int w
   if (keylen < 40 / 8)	/* We want at least 40 bits. */
     return GPG_ERR_INV_KEYLEN;
 
-  if (keylen > 128)
-    return GPG_ERR_INV_KEYLEN;
-
   S = (unsigned char *) ctx->S;
 
   for (i = 0; i < keylen; i++)
@@ -266,10 +248,8 @@ setkey_core (void *context, const unsigned char *key, unsigned int keylen, int w
 }
 
 static gpg_err_code_t
-do_setkey (void *context, const unsigned char *key, unsigned int keylen,
-           cipher_bulk_ops_t *bulk_ops)
+do_setkey (void *context, const unsigned char *key, unsigned int keylen)
 {
-  (void)bulk_ops;
   return setkey_core (context, key, keylen, 1);
 }
 
@@ -349,7 +329,7 @@ selftest (void)
 
 
 
-static const gcry_cipher_oid_spec_t oids_rfc2268_40[] =
+static gcry_cipher_oid_spec_t oids_rfc2268_40[] =
   {
     /*{ "1.2.840.113549.3.2", GCRY_CIPHER_MODE_CBC },*/
     /* pbeWithSHAAnd40BitRC2_CBC */
@@ -357,25 +337,8 @@ static const gcry_cipher_oid_spec_t oids_rfc2268_40[] =
     { NULL }
   };
 
-static const gcry_cipher_oid_spec_t oids_rfc2268_128[] =
-  {
-    /* pbeWithSHAAnd128BitRC2_CBC */
-    { "1.2.840.113549.1.12.1.5", GCRY_CIPHER_MODE_CBC },
-    { NULL }
-  };
-
-gcry_cipher_spec_t _gcry_cipher_spec_rfc2268_40 =
-  {
-    GCRY_CIPHER_RFC2268_40, {0, 0},
-    "RFC2268_40", NULL, oids_rfc2268_40,
-    RFC2268_BLOCKSIZE, 40, sizeof(RFC2268_context),
-    do_setkey, encrypt_block, decrypt_block
-  };
-
-gcry_cipher_spec_t _gcry_cipher_spec_rfc2268_128 =
-  {
-    GCRY_CIPHER_RFC2268_128, {0, 0},
-    "RFC2268_128", NULL, oids_rfc2268_128,
-    RFC2268_BLOCKSIZE, 128, sizeof(RFC2268_context),
-    do_setkey, encrypt_block, decrypt_block
-  };
+gcry_cipher_spec_t _gcry_cipher_spec_rfc2268_40 = {
+  "RFC2268_40", NULL, oids_rfc2268_40,
+  RFC2268_BLOCKSIZE, 40, sizeof(RFC2268_context),
+  do_setkey, do_encrypt, do_decrypt
+};
