@@ -1,6 +1,6 @@
 /* random-csprng.c - CSPRNG style random number generator (libgcrypt classic)
  * Copyright (C) 1998, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
- *               2007, 2008, 2010, 2012  Free Software Foundation, Inc.
+ *               2007, 2008, 2010  Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
@@ -56,10 +56,11 @@
 #include <process.h>
 #endif
 #include "g10lib.h"
+#include "../cipher/rmd.h"
 #include "random.h"
 #include "rand-internal.h"
-#include "cipher.h"         /* _gcry_sha1_hash_buffer  */
-#include "../cipher/sha1.h" /* _gcry_sha1_mixblock     */
+#include "cipher.h" /* Required for the rmd160_hash_buffer() prototype.  */
+#include "ath.h"
 
 #ifndef RAND_MAX   /* For SunOS. */
 #define RAND_MAX 32767
@@ -83,7 +84,7 @@
 
 /* Contstants pertaining to the hash pool. */
 #define BLOCKLEN  64   /* Hash this amount of bytes... */
-#define DIGESTLEN 20   /* ... into a digest of this length (sha-1). */
+#define DIGESTLEN 20   /* ... into a digest of this length (rmd160). */
 /* POOLBLOCKS is the number of digests which make up the pool.  */
 #define POOLBLOCKS 30
 /* POOLSIZE must be a multiple of the digest length to make the AND
@@ -115,7 +116,7 @@ static size_t pool_writepos;
 static size_t pool_readpos;
 
 /* This flag is set to true as soon as the pool has been completely
-   filled the first time.  This may happen either by reading a seed
+   filled the first time.  This may happen either by rereading a seed
    file or by adding enough entropy.  */
 static int pool_filled;
 
@@ -140,7 +141,7 @@ static int pool_balance;
 static int just_mixed;
 
 /* The name of the seed file or NULL if no seed file has been defined.
-   The seed file needs to be registered at initialization time.  We
+   The seed file needs to be regsitered at initialiation time.  We
    keep a malloced copy here.  */
 static char *seed_file_name;
 
@@ -153,7 +154,7 @@ static int allow_seed_file_update;
 static int secure_alloc;
 
 /* This function pointer is set to the actual entropy gathering
-   function during initialization.  After initialization it is
+   function during initailization.  After initialization it is
    guaranteed to point to function.  (On systems without a random
    gatherer module a dummy function is used).*/
 static int (*slow_gather_fnc)(void (*)(const void*, size_t,
@@ -173,8 +174,14 @@ static void (*fast_gather_fnc)(void (*)(const void*, size_t,
    used by regular applications.  */
 static int quick_test;
 
+/* On systems without entropy gathering modules, this flag is set to
+   indicate that the random generator is not working properly.  A
+   warning message is issued as well.  This is useful only for
+   debugging and during development.  */
+static int faked_rng;
+
 /* This is the lock we use to protect all pool operations.  */
-GPGRT_LOCK_DEFINE (pool_lock);
+static ath_mutex_t pool_lock = ATH_MUTEX_INITIALIZER;
 
 /* This is a helper for assert calls.  These calls are used to assert
    that functions are called in a locked state.  It is not meant to be
@@ -182,6 +189,10 @@ GPGRT_LOCK_DEFINE (pool_lock);
    test suite.  */
 static int pool_is_locked;
 
+/* This is the lock we use to protect the buffer used by the nonce
+   generation.  */
+static ath_mutex_t nonce_buffer_lock = ATH_MUTEX_INITIALIZER;
+
 
 /* We keep some counters in this structure for the sake of the
    _gcry_random_dump_stats () function.  */
@@ -235,6 +246,8 @@ static void (*getfnc_fast_random_poll (void))(void (*)(const void*, size_t,
                                               enum random_origins);
 static void read_random_source (enum random_origins origin,
                                 size_t length, int level);
+static int gather_faked (void (*add)(const void*, size_t, enum random_origins),
+                         enum random_origins, size_t length, int level );
 
 
 
@@ -250,10 +263,19 @@ static void
 initialize_basics(void)
 {
   static int initialized;
+  int err;
 
   if (!initialized)
     {
       initialized = 1;
+      err = ath_mutex_init (&pool_lock);
+      if (err)
+        log_fatal ("failed to create the pool lock: %s\n", strerror (err) );
+
+      err = ath_mutex_init (&nonce_buffer_lock);
+      if (err)
+        log_fatal ("failed to create the nonce buffer lock: %s\n",
+                   strerror (err) );
 
 #ifdef USE_RANDOM_DAEMON
       _gcry_daemon_initialize_basics ();
@@ -273,9 +295,9 @@ lock_pool (void)
 {
   int err;
 
-  err = gpgrt_lock_lock (&pool_lock);
+  err = ath_mutex_lock (&pool_lock);
   if (err)
-    log_fatal ("failed to acquire the pool lock: %s\n", gpg_strerror (err));
+    log_fatal ("failed to acquire the pool lock: %s\n", strerror (err));
   pool_is_locked = 1;
 }
 
@@ -286,9 +308,9 @@ unlock_pool (void)
   int err;
 
   pool_is_locked = 0;
-  err = gpgrt_lock_unlock (&pool_lock);
+  err = ath_mutex_unlock (&pool_lock);
   if (err)
-    log_fatal ("failed to release the pool lock: %s\n", gpg_strerror (err));
+    log_fatal ("failed to release the pool lock: %s\n", strerror (err));
 }
 
 
@@ -309,15 +331,20 @@ initialize(void)
          use this extra space (which is allocated in secure memory) as
          a temporary hash buffer */
       rndpool = (secure_alloc
-                 ? xcalloc_secure (1, POOLSIZE + BLOCKLEN)
-                 : xcalloc (1, POOLSIZE + BLOCKLEN));
+                 ? gcry_xcalloc_secure (1, POOLSIZE + BLOCKLEN)
+                 : gcry_xcalloc (1, POOLSIZE + BLOCKLEN));
       keypool = (secure_alloc
-                 ? xcalloc_secure (1, POOLSIZE + BLOCKLEN)
-                 : xcalloc (1, POOLSIZE + BLOCKLEN));
+                 ? gcry_xcalloc_secure (1, POOLSIZE + BLOCKLEN)
+                 : gcry_xcalloc (1, POOLSIZE + BLOCKLEN));
 
       /* Setup the slow entropy gathering function.  The code requires
          that this function exists. */
       slow_gather_fnc = getfnc_gather_random ();
+      if (!slow_gather_fnc)
+        {
+          faked_rng = 1;
+          slow_gather_fnc = gather_faked;
+	}
 
       /* Setup the fast entropy gathering function.  */
       fast_gather_fnc = getfnc_fast_random_poll ();
@@ -343,20 +370,6 @@ _gcry_rngcsprng_initialize (int full)
 }
 
 
-/* Try to close the FDs of the random gather module.  This is
-   currently only implemented for rndlinux. */
-void
-_gcry_rngcsprng_close_fds (void)
-{
-  lock_pool ();
-#if USE_RNDLINUX
-  _gcry_rndlinux_gather_random (NULL, 0, 0, 0);
-  pool_filled = 0; /* Force re-open on next use.  */
-#endif
-  unlock_pool ();
-}
-
-
 void
 _gcry_rngcsprng_dump_stats (void)
 {
@@ -440,7 +453,7 @@ _gcry_rngcsprng_is_faked (void)
   /* We need to initialize due to the runtime determination of
      available entropy gather modules.  */
   initialize();
-  return quick_test;
+  return (faked_rng || quick_test);
 }
 
 
@@ -548,46 +561,41 @@ _gcry_rngcsprng_randomize (void *buffer, size_t length,
 
 
 /*
- * Mix the 600 byte pool.  Note that the 64 byte scratch area directly
- * follows the pool.  The numbers in the diagram give the number of
- * bytes.
- *         <................600...............>   <.64.>
- * pool   |------------------------------------| |------|
- *         <20><.24.>                      <20>
- *          |     |                         +-----+
- *          +-----|-------------------------------|-+
- *                +-------------------------------|-|-+
- *                                                v v v
- *                                               |------|
- *                                                <hash>
- *          +---------------------------------------+
- *          v
- *         <20>
- * pool'  |------------------------------------|
- *         <20><20><.24.>
- *          +---|-----|---------------------------+
- *              +-----|---------------------------|-+
- *                    +---------------------------|-|-+
- *                                                v v v
- *                                               |------|
- *                                                <hash>
- *                                                  |
- *              +-----------------------------------+
- *              v
- *             <20>
- * pool'' |------------------------------------|
- *         <20><20><20><.24.>
- *              +---|-----|-----------------------+
- *                  +-----|-----------------------|-+
- *                        +-----------------------|-|-+
- *                                                v v v
- *
- * and so on until we did this for all 30 blocks.
- *
- * To better protect against implementation errors in this code, we
- * xor a digest of the entire pool into the pool before mixing.
- *
- * Note: this function must only be called with a locked pool.
+   Mix the pool:
+
+   |........blocks*20byte........|20byte|..44byte..|
+   <..44byte..>           <20byte>
+        |                    |
+        |                    +------+
+        +---------------------------|----------+
+                                    v          v
+   |........blocks*20byte........|20byte|..44byte..|
+                                 <.....64bytes.....>
+                                         |
+      +----------------------------------+
+     Hash
+      v
+   |.............................|20byte|..44byte..|
+   <20byte><20byte><..44byte..>
+      |                |
+      |                +---------------------+
+      +-----------------------------+        |
+                                    v        v
+   |.............................|20byte|..44byte..|
+                                 <.....64byte......>
+                                        |
+              +-------------------------+
+             Hash
+              v
+   |.............................|20byte|..44byte..|
+   <20byte><20byte><..44byte..>
+
+   and so on until we did this for all blocks.
+
+   To better protect against implementation errors in this code, we
+   xor a digest of the entire pool into the pool before mixing.
+
+   Note: this function must only be called with a locked pool.
  */
 static void
 mix_pool(unsigned char *pool)
@@ -598,40 +606,41 @@ mix_pool(unsigned char *pool)
   unsigned char *hashbuf = pool + POOLSIZE;
   unsigned char *p, *pend;
   int i, n;
-  SHA1_CONTEXT md;
-  unsigned int nburn;
+  RMD160_CONTEXT md;
 
 #if DIGESTLEN != 20
-#error must have a digest length of 20 for SHA-1
+#error must have a digest length of 20 for ripe-md-160
 #endif
 
   gcry_assert (pool_is_locked);
-  _gcry_sha1_mixblock_init (&md);
+  _gcry_rmd160_init( &md );
 
-  /* pool_0 -> pool'.  */
+  /* Loop over the pool.  */
   pend = pool + POOLSIZE;
-  memcpy (hashbuf, pend - DIGESTLEN, DIGESTLEN);
-  memcpy (hashbuf+DIGESTLEN, pool, BLOCKLEN-DIGESTLEN);
-  nburn = _gcry_sha1_mixblock (&md, hashbuf);
-  memcpy (pool, hashbuf, DIGESTLEN);
+  memcpy(hashbuf, pend - DIGESTLEN, DIGESTLEN );
+  memcpy(hashbuf+DIGESTLEN, pool, BLOCKLEN-DIGESTLEN);
+  _gcry_rmd160_mixblock( &md, hashbuf);
+  memcpy(pool, hashbuf, 20 );
 
   if (failsafe_digest_valid && pool == rndpool)
     {
-      for (i=0; i < DIGESTLEN; i++)
+      for (i=0; i < 20; i++)
         pool[i] ^= failsafe_digest[i];
     }
 
-  /* Loop for the remaining iterations.  */
   p = pool;
   for (n=1; n < POOLBLOCKS; n++)
     {
-      if (p + BLOCKLEN < pend)
-        memcpy (hashbuf, p, BLOCKLEN);
+      memcpy (hashbuf, p, DIGESTLEN);
+
+      p += DIGESTLEN;
+      if (p+DIGESTLEN+BLOCKLEN < pend)
+        memcpy (hashbuf+DIGESTLEN, p+DIGESTLEN, BLOCKLEN-DIGESTLEN);
       else
         {
-          unsigned char *pp = p;
+          unsigned char *pp = p + DIGESTLEN;
 
-          for (i=0; i < BLOCKLEN; i++ )
+          for (i=DIGESTLEN; i < BLOCKLEN; i++ )
             {
               if ( pp >= pend )
                 pp = pool;
@@ -639,22 +648,21 @@ mix_pool(unsigned char *pool)
 	    }
 	}
 
-      _gcry_sha1_mixblock (&md, hashbuf);
-      p += DIGESTLEN;
-      memcpy (p, hashbuf, DIGESTLEN);
+      _gcry_rmd160_mixblock ( &md, hashbuf);
+      memcpy(p, hashbuf, 20 );
     }
 
-  /* Our hash implementation does only leave small parts (64 bytes)
-     of the pool on the stack, so it is okay not to require secure
-     memory here.  Before we use this pool, it will be copied to the
-     help buffer anyway. */
-  if ( pool == rndpool)
-    {
-      _gcry_sha1_hash_buffer (failsafe_digest, pool, POOLSIZE);
-      failsafe_digest_valid = 1;
-    }
+    /* Our hash implementation does only leave small parts (64 bytes)
+       of the pool on the stack, so it is okay not to require secure
+       memory here.  Before we use this pool, it will be copied to the
+       help buffer anyway. */
+    if ( pool == rndpool)
+      {
+        _gcry_rmd160_hash_buffer (failsafe_digest, pool, POOLSIZE);
+        failsafe_digest_valid = 1;
+      }
 
-  _gcry_burn_stack (nburn);
+    _gcry_burn_stack (384); /* for the rmd160_mixblock(), rmd160_hash_buffer */
 }
 
 
@@ -663,89 +671,7 @@ _gcry_rngcsprng_set_seed_file (const char *name)
 {
   if (seed_file_name)
     BUG ();
-  seed_file_name = xstrdup (name);
-}
-
-
-
-/* Helper for my_open.
- * Return a malloced wide char string from an UTF-8 encoded input
- * string STRING.  Caller must free this value.  Returns NULL and sets
- * ERRNO on failure.  Calling this function with STRING set to NULL is
- * not defined.  */
-#ifdef HAVE_W32_SYSTEM
-static wchar_t *
-utf8_to_wchar (const char *string)
-{
-  int n;
-  size_t nbytes;
-  wchar_t *result;
-
-  n = MultiByteToWideChar (CP_UTF8, 0, string, -1, NULL, 0);
-  if (n < 0)
-    {
-      gpg_err_set_errno (EINVAL);
-      return NULL;
-    }
-
-  nbytes = (size_t)(n+1) * sizeof(*result);
-  if (nbytes / sizeof(*result) != (n+1))
-    {
-      gpg_err_set_errno (ENOMEM);
-      return NULL;
-    }
-  result = xtrymalloc (nbytes);
-  if (!result)
-    return NULL;
-
-  n = MultiByteToWideChar (CP_UTF8, 0, string, -1, result, n);
-  if (n < 0)
-    {
-      xfree (result);
-      gpg_err_set_errno (EINVAL);
-      result = NULL;
-    }
-  return result;
-}
-#endif /*HAVE_W32_SYSTEM*/
-
-
-/* Helper for my_open.  */
-#ifdef HAVE_W32_SYSTEM
-static int
-any8bitchar (const char *string)
-{
-  if (string)
-    for ( ; *string; string++)
-      if ((*string & 0x80))
-        return 1;
-  return 0;
-}
-#endif /*HAVE_W32_SYSTEM*/
-
-
-/* A wrapper around open to handle Unicode file names under Windows.  */
-static int
-my_open (const char *name, int flags, unsigned int mode)
-{
-#ifdef HAVE_W32_SYSTEM
-  if (any8bitchar (name))
-    {
-      wchar_t *wname;
-      int ret;
-
-      wname = utf8_to_wchar (name);
-      if (!wname)
-        return -1;
-      ret = _wopen (wname, flags, mode);
-      xfree (wname);
-      return ret;
-    }
-  else
-    return open (name, flags, mode);
-#else
-  return open (name, flags, mode);
-#endif
+  seed_file_name = gcry_xstrdup (name);
 }
 
 
@@ -786,10 +712,6 @@ lock_seed_file (int fd, const char *fname, int for_write)
       if (backoff < 10)
         backoff++ ;
     }
-#else
-  (void)fd;
-  (void)fname;
-  (void)for_write;
 #endif /*!LOCK_SEED_FILE*/
   return 0;
 }
@@ -803,12 +725,12 @@ lock_seed_file (int fd, const char *fname, int for_write)
    out the same pool and then race for updating it (the last update
    overwrites earlier updates).  They will differentiate only by the
    weak entropy that is added in read_seed_file based on the PID and
-   clock, and up to 32 bytes from a non-blocking entropy source.  The
+   clock, and up to 16 bytes of weak random non-blockingly.  The
    consequence is that the output of these different instances is
    correlated to some extent.  In the perfect scenario, the attacker
    can control (or at least guess) the PID and clock of the
    application, and drain the system's entropy pool to reduce the "up
-   to 32 bytes" above to 0.  Then the dependencies of the initial
+   to 16 bytes" above to 0.  Then the dependencies of the initial
    states of the pools are completely known.  */
 static int
 read_seed_file (void)
@@ -824,9 +746,9 @@ read_seed_file (void)
     return 0;
 
 #ifdef HAVE_DOSISH_SYSTEM
-  fd = my_open (seed_file_name, O_RDONLY | O_BINARY, 0);
+  fd = open( seed_file_name, O_RDONLY | O_BINARY );
 #else
-  fd = my_open (seed_file_name, O_RDONLY, 0);
+  fd = open( seed_file_name, O_RDONLY );
 #endif
   if( fd == -1 && errno == ENOENT)
     {
@@ -900,16 +822,12 @@ read_seed_file (void)
     add_randomness( &x, sizeof(x), RANDOM_ORIGIN_INIT );
   }
 
-  /* And read a few bytes from our entropy source.  If we have the
-   * Jitter RNG we can fast get a lot of entropy.  Thus we read 1024
-   * bits from that source.
-   *
-   * Without the Jitter RNG we keep the old method of reading only a
-   * few bytes usually from /dev/urandom which won't block.  */
-  if (_gcry_rndjent_get_version (NULL))
-    read_random_source (RANDOM_ORIGIN_INIT, 128, GCRY_STRONG_RANDOM);
-  else
-    read_random_source (RANDOM_ORIGIN_INIT, 32, GCRY_STRONG_RANDOM);
+  /* And read a few bytes from our entropy source.  By using a level
+   * of 0 this will not block and might not return anything with some
+   * entropy drivers, however the rndlinux driver will use
+   * /dev/urandom and return some stuff - Do not read too much as we
+   * want to be friendly to the scare system entropy resource. */
+  read_random_source ( RANDOM_ORIGIN_INIT, 16, GCRY_WEAK_RANDOM );
 
   allow_seed_file_update = 1;
   return 1;
@@ -946,7 +864,7 @@ _gcry_rngcsprng_update_seed_file (void)
 
 
   /* Copy the entropy pool to a scratch pool and mix both of them. */
-  for (i=0,dp=(unsigned long*)(void*)keypool, sp=(unsigned long*)(void*)rndpool;
+  for (i=0,dp=(unsigned long*)keypool, sp=(unsigned long*)rndpool;
        i < POOLWORDS; i++, dp++, sp++ )
     {
       *dp = *sp + ADD_VALUE;
@@ -955,13 +873,13 @@ _gcry_rngcsprng_update_seed_file (void)
   mix_pool(keypool); rndstats.mixkey++;
 
 #if defined(HAVE_DOSISH_SYSTEM) || defined(__CYGWIN__)
-  fd = my_open (seed_file_name, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,
-                S_IRUSR|S_IWUSR );
+  fd = open (seed_file_name, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,
+             S_IRUSR|S_IWUSR );
 #else
 # if LOCK_SEED_FILE
-    fd = my_open (seed_file_name, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR );
+    fd = open (seed_file_name, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR );
 # else
-    fd = my_open (seed_file_name, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR );
+    fd = open (seed_file_name, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR );
 # endif
 #endif
 
@@ -1055,8 +973,8 @@ read_pool (byte *buffer, size_t length, int level)
 
       pool_balance = 0;
       needed = length - pool_balance;
-      if (needed < 16)  /* At least 128 bits.  */
-        needed = 16;
+      if (needed < POOLSIZE/2)
+        needed = POOLSIZE/2;
       else if( needed > POOLSIZE )
         BUG ();
       read_random_source (RANDOM_ORIGIN_EXTRAPOLL, needed,
@@ -1102,7 +1020,7 @@ read_pool (byte *buffer, size_t length, int level)
     }
 
   /* Create a new pool. */
-  for(i=0,dp=(unsigned long*)(void*)keypool, sp=(unsigned long*)(void*)rndpool;
+  for(i=0,dp=(unsigned long*)keypool, sp=(unsigned long*)rndpool;
       i < POOLWORDS; i++, dp++, sp++ )
     *dp = *sp + ADD_VALUE;
 
@@ -1316,7 +1234,7 @@ do_fast_random_poll (void)
 # endif /*!RUSAGE_SELF*/
 #endif /*HAVE_GETRUSAGE*/
 
-  /* Time and clock are available on all systems - so we better do it
+  /* Time and clock are availabe on all systems - so we better do it
      just in case one of the above functions didn't work.  */
   {
     time_t x = time(NULL);
@@ -1357,11 +1275,134 @@ _gcry_rngcsprng_fast_poll (void)
 
 
 static void
-read_random_source (enum random_origins origin, size_t length, int level)
+read_random_source (enum random_origins orgin, size_t length, int level )
 {
   if ( !slow_gather_fnc )
     log_fatal ("Slow entropy gathering module not yet initialized\n");
 
-  if (slow_gather_fnc (add_randomness, origin, length, level) < 0)
+  if ( slow_gather_fnc (add_randomness, orgin, length, level) < 0)
     log_fatal ("No way to gather entropy for the RNG\n");
 }
+
+
+static int
+gather_faked (void (*add)(const void*, size_t, enum random_origins),
+              enum random_origins origin, size_t length, int level )
+{
+  static int initialized=0;
+  size_t n;
+  char *buffer, *p;
+
+  (void)add;
+  (void)level;
+
+  if ( !initialized )
+    {
+      log_info(_("WARNING: using insecure random number generator!!\n"));
+      initialized=1;
+#ifdef HAVE_RAND
+      srand( time(NULL)*getpid());
+#else
+      srandom( time(NULL)*getpid());
+#endif
+    }
+
+  p = buffer = gcry_xmalloc( length );
+  n = length;
+#ifdef HAVE_RAND
+  while ( n-- )
+    *p++ = ((unsigned)(1 + (int) (256.0*rand()/(RAND_MAX+1.0)))-1);
+#else
+  while ( n-- )
+    *p++ = ((unsigned)(1 + (int) (256.0*random()/(RAND_MAX+1.0)))-1);
+#endif
+  add_randomness ( buffer, length, origin );
+  gcry_free (buffer);
+  return 0; /* okay */
+}
+
+
+/* Create an unpredicable nonce of LENGTH bytes in BUFFER. */
+void
+_gcry_rngcsprng_create_nonce (void *buffer, size_t length)
+{
+  static unsigned char nonce_buffer[20+8];
+  static int nonce_buffer_initialized = 0;
+  static volatile pid_t my_pid; /* The volatile is there to make sure the
+                                   compiler does not optimize the code away
+                                   in case the getpid function is badly
+                                   attributed. */
+  volatile pid_t apid;
+  unsigned char *p;
+  size_t n;
+  int err;
+
+  /* Make sure we are initialized. */
+  initialize ();
+
+#ifdef USE_RANDOM_DAEMON
+  if (allow_daemon
+      && !_gcry_daemon_create_nonce (daemon_socket_name, buffer, length))
+    return; /* The daemon succeeded. */
+  allow_daemon = 0; /* Daemon failed - switch off. */
+#endif /*USE_RANDOM_DAEMON*/
+
+  /* Acquire the nonce buffer lock. */
+  err = ath_mutex_lock (&nonce_buffer_lock);
+  if (err)
+    log_fatal ("failed to acquire the nonce buffer lock: %s\n",
+               strerror (err));
+
+  apid = getpid ();
+  /* The first time initialize our buffer. */
+  if (!nonce_buffer_initialized)
+    {
+      time_t atime = time (NULL);
+      pid_t xpid = apid;
+
+      my_pid = apid;
+
+      if ((sizeof apid + sizeof atime) > sizeof nonce_buffer)
+        BUG ();
+
+      /* Initialize the first 20 bytes with a reasonable value so that
+         a failure of gcry_randomize won't affect us too much.  Don't
+         care about the uninitialized remaining bytes. */
+      p = nonce_buffer;
+      memcpy (p, &xpid, sizeof xpid);
+      p += sizeof xpid;
+      memcpy (p, &atime, sizeof atime);
+
+      /* Initialize the never changing private part of 64 bits. */
+      gcry_randomize (nonce_buffer+20, 8, GCRY_WEAK_RANDOM);
+
+      nonce_buffer_initialized = 1;
+    }
+  else if ( my_pid != apid )
+    {
+      /* We forked. Need to reseed the buffer - doing this for the
+         private part should be sufficient. */
+      gcry_randomize (nonce_buffer+20, 8, GCRY_WEAK_RANDOM);
+      /* Update the pid so that we won't run into here again and
+         again. */
+      my_pid = apid;
+    }
+
+  /* Create the nonce by hashing the entire buffer, returning the hash
+     and updating the first 20 bytes of the buffer with this hash. */
+  for (p = buffer; length > 0; length -= n, p += n)
+    {
+      _gcry_sha1_hash_buffer (nonce_buffer,
+                              nonce_buffer, sizeof nonce_buffer);
+      n = length > 20? 20 : length;
+      memcpy (p, nonce_buffer, n);
+    }
+
+
+  /* Release the nonce buffer lock. */
+  err = ath_mutex_unlock (&nonce_buffer_lock);
+  if (err)
+    log_fatal ("failed to release the nonce buffer lock: %s\n",
+               strerror (err));
+
+}
